package Fb::Cache;

use 5.006;
use strict;
use warnings;

require Exporter;
our @ISA=qw(Exporter);

sub new {
	my ($package,$options)=@_;
	if ( ! $options ) {
		$options={};
	}
	my $self={
		default_expires_in => 3600,
		check_interval => 180,
		data => {},
		lastcleanup => 0,
	};
	if ( $options->{default_expires_in} ) {
		$self->{default_expires_in}=$options->{default_expires_in};
	}
	if ( $options->{check_interval} ) {
		$self->{check_interval}=$options->{check_interval};
	}
	my $me=bless($self,$package);
}


sub set {
	my ($self,$key,$value)=@_;
	if ( ref($value) eq 'CODE' ) {
		$value=&{$value}();
	}
	$self->{data}->{$key}=[time(),$value];
	return $value;
}

sub remove {
	my ($self,$key)=@_;
	delete $self->{data}->{$key};
}

sub removePattern {
	my ($self,$pcre)=@_;
	my @keys=grep { /^$pcre$/ } keys %{$self->{data}};
	my $count=scalar(@keys);
	for my $key (@keys) {
		delete $self->{data}->{$key};
	}
	return $count;
}

sub clear {
	my $self=$_[0];
	$self->{data}={};
	$self->{lastcleanup}=time();
	return 1;
}

sub keys {
	return [keys %{$_[0]->{data}}];
}

sub cleanup {
	my $self=shift;
	my $expire=time()-$self->{default_expires_in};
	my $count=0;
	while (my ($key,$item) = each %{$self->{data}}) {
		if ( $item->[0] < $expire ) {
			delete $self->{data}->{$key};
			$count++
		}
	}
	return $count;
}

sub get {
	my ($self,$key,%options)=@_;
	my $item=$self->{data}->{$key};
	my $undef=0;
	my $now=time();
	my $expire=$now-$self->{default_expires_in};
	if ( ! $options{nocleanup} ) {
		if ( ( $now - $self->{lastcleanup} ) > $self->{check_interval} ) {
			$self->cleanup();
			$self->{lastcleanup}=$now;
		}
	}
	
	if ( ! defined($item) ) {
		$undef=1;
	} else {
		if ( $item->[0] < $expire ) {
			delete $self->{data}->{$key};
			$undef=1;
		}
	}
	if ( $undef ) {
		if ( defined($options{create}) ) {
			return $self->set($key,$options{create});
		} else {
			return undef;
		}
	}
	return $item->[1];
}

1;
